@@ -34,9 +34,14 @@
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 
+#include <gconf/gconf-client.h>
+
 #include "gdm-user-manager.h"
 #include "gdm-user-chooser-widget.h"
 
+
+#define KEY_DISABLE_USER_LIST "/apps/gdm/simple-greeter/disable_user_list"
+
 enum {
         USER_NO_DISPLAY              = 1 << 0,
         USER_ACCOUNT_DISABLED        = 1 << 1,
@@ -46,7 +51,7 @@ enum {
 
 #define GDM_USER_CHOOSER_WIDGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_USER_CHOOSER_WIDGET, GdmUserChooserWidgetPrivate))
 
-#define ICON_SIZE 64
+#define MAX_ICON_SIZE 128
 
 struct GdmUserChooserWidgetPrivate
 {
@@ -56,13 +61,20 @@ struct GdmUserChooserWidgetPrivate
         GdkPixbuf      *logged_in_pixbuf;
         GdkPixbuf      *stock_person_pixbuf;
 
-        guint           show_other_user : 1;
-        guint           show_guest_user : 1;
-        guint           show_auto_user : 1;
+        guint           loaded : 1;
+        guint           show_user_other : 1;
+        guint           show_user_guest : 1;
+        guint           show_user_auto : 1;
+        guint           show_normal_users : 1;
+
+        guint           load_idle_id;
 };
 
 enum {
         PROP_0,
+        PROP_SHOW_USER_GUEST,
+        PROP_SHOW_USER_AUTO,
+        PROP_SHOW_USER_OTHER,
 };
 
 static void     gdm_user_chooser_widget_class_init  (GdmUserChooserWidgetClass *klass);
@@ -71,6 +83,45 @@ static void     gdm_user_chooser_widget_finalize    (GObject                   *
 
 G_DEFINE_TYPE (GdmUserChooserWidget, gdm_user_chooser_widget, GDM_TYPE_CHOOSER_WIDGET)
 
+static int
+get_font_height_for_widget (GtkWidget *widget)
+{
+        PangoFontMetrics *metrics;
+        PangoContext     *context;
+        int               ascent;
+        int               descent;
+        int               height;
+
+        gtk_widget_ensure_style (widget);
+        context = gtk_widget_get_pango_context (widget);
+        metrics = pango_context_get_metrics (context,
+                                             widget->style->font_desc,
+                                             pango_context_get_language (context));
+
+        ascent = pango_font_metrics_get_ascent (metrics);
+        descent = pango_font_metrics_get_descent (metrics);
+        height = PANGO_PIXELS (ascent + descent);
+        pango_font_metrics_unref (metrics);
+        return height;
+}
+
+static int
+get_icon_height_for_widget (GtkWidget *widget)
+{
+        int font_height;
+        int height;
+
+        font_height = get_font_height_for_widget (widget);
+        height = 3 * font_height;
+        if (height > MAX_ICON_SIZE) {
+                height = MAX_ICON_SIZE;
+        }
+
+        g_debug ("GdmUserChooserWidget: font height %d; using icon size %d", font_height, height);
+
+        return height;
+}
+
 static void
 add_user_other (GdmUserChooserWidget *widget)
 {
@@ -132,13 +183,13 @@ remove_user_auto (GdmUserChooserWidget *widget)
 }
 
 void
-gdm_user_chooser_widget_set_show_other_user (GdmUserChooserWidget *widget,
+gdm_user_chooser_widget_set_show_user_other (GdmUserChooserWidget *widget,
                                              gboolean              show_user)
 {
         g_return_if_fail (GDM_IS_USER_CHOOSER_WIDGET (widget));
 
-        if (widget->priv->show_other_user != show_user) {
-                widget->priv->show_other_user = show_user;
+        if (widget->priv->show_user_other != show_user) {
+                widget->priv->show_user_other = show_user;
                 if (show_user) {
                         add_user_other (widget);
                 } else {
@@ -148,13 +199,13 @@ gdm_user_chooser_widget_set_show_other_user (GdmUserChooserWidget *widget,
 }
 
 void
-gdm_user_chooser_widget_set_show_guest_user (GdmUserChooserWidget *widget,
+gdm_user_chooser_widget_set_show_user_guest (GdmUserChooserWidget *widget,
                                              gboolean              show_user)
 {
         g_return_if_fail (GDM_IS_USER_CHOOSER_WIDGET (widget));
 
-        if (widget->priv->show_guest_user != show_user) {
-                widget->priv->show_guest_user = show_user;
+        if (widget->priv->show_user_guest != show_user) {
+                widget->priv->show_user_guest = show_user;
                 if (show_user) {
                         add_user_guest (widget);
                 } else {
@@ -164,13 +215,13 @@ gdm_user_chooser_widget_set_show_guest_user (GdmUserChooserWidget *widget,
 }
 
 void
-gdm_user_chooser_widget_set_show_auto_user (GdmUserChooserWidget *widget,
+gdm_user_chooser_widget_set_show_user_auto (GdmUserChooserWidget *widget,
                                             gboolean              show_user)
 {
         g_return_if_fail (GDM_IS_USER_CHOOSER_WIDGET (widget));
 
-        if (widget->priv->show_auto_user != show_user) {
-                widget->priv->show_auto_user = show_user;
+        if (widget->priv->show_user_auto != show_user) {
+                widget->priv->show_user_auto = show_user;
                 if (show_user) {
                         add_user_auto (widget);
                 } else {
@@ -211,7 +262,20 @@ gdm_user_chooser_widget_set_property (GObject        *object,
                                       const GValue   *value,
                                       GParamSpec     *pspec)
 {
+        GdmUserChooserWidget *self;
+
+        self = GDM_USER_CHOOSER_WIDGET (object);
+
         switch (prop_id) {
+        case PROP_SHOW_USER_AUTO:
+                gdm_user_chooser_widget_set_show_user_auto (self, g_value_get_boolean (value));
+                break;
+        case PROP_SHOW_USER_GUEST:
+                gdm_user_chooser_widget_set_show_user_guest (self, g_value_get_boolean (value));
+                break;
+        case PROP_SHOW_USER_OTHER:
+                gdm_user_chooser_widget_set_show_user_other (self, g_value_get_boolean (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -224,30 +288,228 @@ gdm_user_chooser_widget_get_property (GObject        *object,
                                       GValue         *value,
                                       GParamSpec     *pspec)
 {
+        GdmUserChooserWidget *self;
+
+        self = GDM_USER_CHOOSER_WIDGET (object);
+
         switch (prop_id) {
+        case PROP_SHOW_USER_AUTO:
+                g_value_set_boolean (value, self->priv->show_user_auto);
+                break;
+        case PROP_SHOW_USER_GUEST:
+                g_value_set_boolean (value, self->priv->show_user_guest);
+                break;
+        case PROP_SHOW_USER_OTHER:
+                g_value_set_boolean (value, self->priv->show_user_other);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
         }
 }
 
+static gboolean
+is_user_list_disabled (GdmUserChooserWidget *widget)
+{
+        GConfClient *client;
+        GError      *error;
+        gboolean     result;
+
+        client = gconf_client_get_default ();
+        error = NULL;
+        result = gconf_client_get_bool (client, KEY_DISABLE_USER_LIST, &error);
+        if (error != NULL) {
+                g_debug ("GdmUserChooserWidget: unable to get disable-user-list configuration: %s", error->message);
+                g_error_free (error);
+        }
+        g_object_unref (client);
+
+        return result;
+}
+
+static void
+add_user (GdmUserChooserWidget *widget,
+          GdmUser              *user)
+{
+        GdkPixbuf    *pixbuf;
+        char         *tooltip;
+        gboolean      is_logged_in;
+        int           size;
+
+        if (!widget->priv->show_normal_users) {
+                return;
+        }
+
+        size = get_icon_height_for_widget (widget);
+        pixbuf = gdm_user_render_icon (user, size);
+        if (pixbuf == NULL && widget->priv->stock_person_pixbuf != NULL) {
+                pixbuf = g_object_ref (widget->priv->stock_person_pixbuf);
+        }
+
+        tooltip = g_strdup_printf (_("Log in as %s"),
+                                   gdm_user_get_user_name (user));
+
+        is_logged_in = gdm_user_get_num_sessions (user) > 0;
+
+        g_debug ("GdmUserChooserWidget: User added name:%s logged-in:%d pixbuf:%p",
+                 gdm_user_get_user_name (user),
+                 is_logged_in,
+                 pixbuf);
+
+        gdm_chooser_widget_add_item (GDM_CHOOSER_WIDGET (widget),
+                                     gdm_user_get_user_name (user),
+                                     pixbuf,
+                                     gdm_user_get_real_name (user),
+                                     tooltip,
+                                     gdm_user_get_login_frequency (user),
+                                     is_logged_in,
+                                     FALSE);
+        g_free (tooltip);
+
+        if (pixbuf != NULL) {
+                g_object_unref (pixbuf);
+        }
+}
+
+static void
+on_user_added (GdmUserManager       *manager,
+               GdmUser              *user,
+               GdmUserChooserWidget *widget)
+{
+        /* wait for all users to be loaded */
+        if (! widget->priv->loaded) {
+                return;
+        }
+        add_user (widget, user);
+}
+
+static void
+on_user_removed (GdmUserManager       *manager,
+                 GdmUser              *user,
+                 GdmUserChooserWidget *widget)
+{
+        const char *user_name;
+
+        g_debug ("GdmUserChooserWidget: User removed: %s", gdm_user_get_user_name (user));
+        /* wait for all users to be loaded */
+        if (! widget->priv->loaded) {
+                return;
+        }
+
+        user_name = gdm_user_get_user_name (user);
+
+        gdm_chooser_widget_remove_item (GDM_CHOOSER_WIDGET (widget),
+                                        user_name);
+}
+
+static void
+on_user_is_logged_in_changed (GdmUserManager       *manager,
+                              GdmUser              *user,
+                              GdmUserChooserWidget *widget)
+{
+        const char *user_name;
+        gboolean    is_logged_in;
+
+        g_debug ("GdmUserChooserWidget: User logged in changed: %s", gdm_user_get_user_name (user));
+
+        user_name = gdm_user_get_user_name (user);
+        is_logged_in = gdm_user_get_num_sessions (user) > 0;
+
+        gdm_chooser_widget_set_item_in_use (GDM_CHOOSER_WIDGET (widget),
+                                            user_name,
+                                            is_logged_in);
+}
+
+static void
+on_user_login_frequency_changed (GdmUserManager       *manager,
+                                 GdmUser              *user,
+                                 GdmUserChooserWidget *widget)
+{
+        const char *user_name;
+        gulong      freq;
+
+        g_debug ("GdmUserChooserWidget: User login frequency changed: %s", gdm_user_get_user_name (user));
+
+        user_name = gdm_user_get_user_name (user);
+        freq = gdm_user_get_login_frequency (user);
+
+        gdm_chooser_widget_set_item_priority (GDM_CHOOSER_WIDGET (widget),
+                                              user_name,
+                                              freq);
+}
+
+static void
+on_users_loaded (GdmUserManager       *manager,
+                 GdmUserChooserWidget *widget)
+{
+        GSList *users;
+
+        widget->priv->loaded = TRUE;
+
+        g_debug ("GdmUserChooserWidget: Users loaded");
+
+        users = gdm_user_manager_list_users (manager);
+        while (users != NULL) {
+                add_user (widget, users->data);
+                users = g_slist_delete_link (users, users);
+        }
+
+        gtk_widget_grab_focus (GTK_WIDGET (widget));
+
+        gdm_chooser_widget_loaded (GDM_CHOOSER_WIDGET (widget));
+}
+
+static gboolean
+load_users (GdmUserChooserWidget *widget)
+{
+
+        if (widget->priv->show_normal_users) {
+                widget->priv->manager = gdm_user_manager_ref_default ();
+                g_signal_connect (widget->priv->manager,
+                                  "user-added",
+                                  G_CALLBACK (on_user_added),
+                                  widget);
+                g_signal_connect (widget->priv->manager,
+                                  "user-removed",
+                                  G_CALLBACK (on_user_removed),
+                                  widget);
+                g_signal_connect (widget->priv->manager,
+                                  "users-loaded",
+                                  G_CALLBACK (on_users_loaded),
+                                  widget);
+                g_signal_connect (widget->priv->manager,
+                                  "user-is-logged-in-changed",
+                                  G_CALLBACK (on_user_is_logged_in_changed),
+                                  widget);
+                g_signal_connect (widget->priv->manager,
+                                  "user-login-frequency-changed",
+                                  G_CALLBACK (on_user_login_frequency_changed),
+                                  widget);
+        } else {
+                gdm_chooser_widget_loaded (GDM_CHOOSER_WIDGET (widget));
+        }
+
+        widget->priv->load_idle_id = 0;
+
+        return FALSE;
+}
+
 static GObject *
 gdm_user_chooser_widget_constructor (GType                  type,
                                      guint                  n_construct_properties,
                                      GObjectConstructParam *construct_properties)
 {
-        GdmUserChooserWidget      *user_chooser_widget;
+        GdmUserChooserWidget      *widget;
 
-        user_chooser_widget = GDM_USER_CHOOSER_WIDGET (G_OBJECT_CLASS (gdm_user_chooser_widget_parent_class)->constructor (type,
-                                                                                                                           n_construct_properties,
-                                                                                                                           construct_properties));
+        widget = GDM_USER_CHOOSER_WIDGET (G_OBJECT_CLASS (gdm_user_chooser_widget_parent_class)->constructor (type,
+                                                                                                              n_construct_properties,
+                                                                                                              construct_properties));
 
-        /* FIXME: make these construct properties */
-        gdm_user_chooser_widget_set_show_guest_user (user_chooser_widget, FALSE);
-        gdm_user_chooser_widget_set_show_auto_user (user_chooser_widget, FALSE);
-        gdm_user_chooser_widget_set_show_other_user (user_chooser_widget, TRUE);
+        widget->priv->show_normal_users = !is_user_list_disabled (widget);
 
-        return G_OBJECT (user_chooser_widget);
+        widget->priv->load_idle_id = g_idle_add ((GSourceFunc)load_users, widget);
+
+        return G_OBJECT (widget);
 }
 
 static void
@@ -259,6 +521,11 @@ gdm_user_chooser_widget_dispose (GObject *object)
 
         G_OBJECT_CLASS (gdm_user_chooser_widget_parent_class)->dispose (object);
 
+        if (widget->priv->load_idle_id > 0) {
+                g_source_remove (widget->priv->load_idle_id);
+                widget->priv->load_idle_id = 0;
+        }
+
         if (widget->priv->logged_in_pixbuf != NULL) {
                 g_object_unref (widget->priv->logged_in_pixbuf);
                 widget->priv->logged_in_pixbuf = NULL;
@@ -273,13 +540,37 @@ gdm_user_chooser_widget_dispose (GObject *object)
 static void
 gdm_user_chooser_widget_class_init (GdmUserChooserWidgetClass *klass)
 {
-        GObjectClass          *object_class = G_OBJECT_CLASS (klass);
+        GObjectClass   *object_class = G_OBJECT_CLASS (klass);
+
         object_class->get_property = gdm_user_chooser_widget_get_property;
         object_class->set_property = gdm_user_chooser_widget_set_property;
         object_class->constructor = gdm_user_chooser_widget_constructor;
         object_class->dispose = gdm_user_chooser_widget_dispose;
         object_class->finalize = gdm_user_chooser_widget_finalize;
 
+
+        g_object_class_install_property (object_class,
+                                         PROP_SHOW_USER_AUTO,
+                                         g_param_spec_boolean ("show-user-auto",
+                                                               "show user auto",
+                                                               "show user auto",
+                                                               FALSE,
+                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+        g_object_class_install_property (object_class,
+                                         PROP_SHOW_USER_GUEST,
+                                         g_param_spec_boolean ("show-user-guest",
+                                                               "show user guest",
+                                                               "show user guest",
+                                                               FALSE,
+                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+        g_object_class_install_property (object_class,
+                                         PROP_SHOW_USER_OTHER,
+                                         g_param_spec_boolean ("show-user-other",
+                                                               "show user other",
+                                                               "show user other",
+                                                               TRUE,
+                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
         g_type_class_add_private (klass, sizeof (GdmUserChooserWidgetPrivate));
 }
 
@@ -287,10 +578,13 @@ static GdkPixbuf *
 get_stock_person_pixbuf (GdmUserChooserWidget *widget)
 {
         GdkPixbuf *pixbuf;
+        int        size;
+
+        size = get_icon_height_for_widget (widget);
 
         pixbuf = gtk_icon_theme_load_icon (widget->priv->icon_theme,
                                            DEFAULT_USER_ICON,
-                                           ICON_SIZE,
+                                           size,
                                            0,
                                            NULL);
 
@@ -301,10 +595,13 @@ static GdkPixbuf *
 get_logged_in_pixbuf (GdmUserChooserWidget *widget)
 {
         GdkPixbuf *pixbuf;
+        int        size;
+
+        size = get_icon_height_for_widget (widget);
 
         pixbuf = gtk_icon_theme_load_icon (widget->priv->icon_theme,
                                            "emblem-default",
-                                           ICON_SIZE / 3,
+                                           size / 3,
                                            0,
                                            NULL);
 
@@ -386,104 +683,6 @@ setup_icons (GdmUserChooserWidget *widget)
         load_icons (widget);
 }
 
-static void
-on_users_loaded (GdmUserManager       *manager,
-                 GdmUserChooserWidget *widget)
-{
-        gdm_chooser_widget_activate_on_one_item (GDM_CHOOSER_WIDGET (widget),
-                                                 TRUE);
-}
-
-static void
-on_user_added (GdmUserManager       *manager,
-               GdmUser              *user,
-               GdmUserChooserWidget *widget)
-{
-        GdkPixbuf    *pixbuf;
-        char         *tooltip;
-        gboolean      is_logged_in;
-
-        pixbuf = gdm_user_render_icon (user, ICON_SIZE);
-        if (pixbuf == NULL && widget->priv->stock_person_pixbuf != NULL) {
-                pixbuf = g_object_ref (widget->priv->stock_person_pixbuf);
-        }
-
-        tooltip = g_strdup_printf (_("Log in as %s"),
-                                   gdm_user_get_user_name (user));
-
-        is_logged_in = gdm_user_get_num_sessions (user) > 0;
-
-        g_debug ("GdmUserChooserWidget: User added name:%s logged-in:%d pixbuf:%p",
-                 gdm_user_get_user_name (user),
-                 is_logged_in,
-                 pixbuf);
-
-        gdm_chooser_widget_add_item (GDM_CHOOSER_WIDGET (widget),
-                                     gdm_user_get_user_name (user),
-                                     pixbuf,
-                                     gdm_user_get_real_name (user),
-                                     tooltip,
-                                     gdm_user_get_login_frequency (user),
-                                     is_logged_in,
-                                     FALSE);
-        g_free (tooltip);
-
-        if (pixbuf != NULL) {
-                g_object_unref (pixbuf);
-        }
-}
-
-static void
-on_user_removed (GdmUserManager       *manager,
-                 GdmUser              *user,
-                 GdmUserChooserWidget *widget)
-{
-        const char *user_name;
-
-        g_debug ("GdmUserChooserWidget: User removed: %s", gdm_user_get_user_name (user));
-
-        user_name = gdm_user_get_user_name (user);
-
-        gdm_chooser_widget_remove_item (GDM_CHOOSER_WIDGET (widget),
-                                        user_name);
-}
-
-static void
-on_user_is_logged_in_changed (GdmUserManager       *manager,
-                              GdmUser              *user,
-                              GdmUserChooserWidget *widget)
-{
-        const char *user_name;
-        gboolean    is_logged_in;
-
-        g_debug ("GdmUserChooserWidget: User logged in changed: %s", gdm_user_get_user_name (user));
-
-        user_name = gdm_user_get_user_name (user);
-        is_logged_in = gdm_user_get_num_sessions (user) > 0;
-
-        gdm_chooser_widget_set_item_in_use (GDM_CHOOSER_WIDGET (widget),
-                                            user_name,
-                                            is_logged_in);
-}
-
-static void
-on_user_login_frequency_changed (GdmUserManager       *manager,
-                                 GdmUser              *user,
-                                 GdmUserChooserWidget *widget)
-{
-        const char *user_name;
-        gulong      freq;
-
-        g_debug ("GdmUserChooserWidget: User login frequency changed: %s", gdm_user_get_user_name (user));
-
-        user_name = gdm_user_get_user_name (user);
-        freq = gdm_user_get_login_frequency (user);
-
-        gdm_chooser_widget_set_item_priority (GDM_CHOOSER_WIDGET (widget),
-                                              user_name,
-                                              freq);
-}
-
 static void
 gdm_user_chooser_widget_init (GdmUserChooserWidget *widget)
 {
@@ -494,28 +693,6 @@ gdm_user_chooser_widget_init (GdmUserChooserWidget *widget)
         gdm_chooser_widget_set_in_use_message (GDM_CHOOSER_WIDGET (widget),
                                                _("Currently logged in"));
 
-        widget->priv->manager = gdm_user_manager_ref_default ();
-        g_signal_connect (widget->priv->manager,
-                          "users-loaded",
-                          G_CALLBACK (on_users_loaded),
-                          widget);
-        g_signal_connect (widget->priv->manager,
-                          "user-added",
-                          G_CALLBACK (on_user_added),
-                          widget);
-        g_signal_connect (widget->priv->manager,
-                          "user-removed",
-                          G_CALLBACK (on_user_removed),
-                          widget);
-        g_signal_connect (widget->priv->manager,
-                          "user-is-logged-in-changed",
-                          G_CALLBACK (on_user_is_logged_in_changed),
-                          widget);
-        g_signal_connect (widget->priv->manager,
-                          "user-login-frequency-changed",
-                          G_CALLBACK (on_user_login_frequency_changed),
-                          widget);
-
         setup_icons (widget);
 }
 
